#include "servo.h"

#include <esp32-hal.h> // For `millis()`

void performReset(void * pvParameters) {
    ServoController* servo = (ServoController *) pvParameters;
    for(;;){
        servo->loop(millis());
        delay(50);
    }
}

ServoController::ServoController(ServoConfiguration configuration) {

    // Create a task that runs on a different core,
    // So that it's always executed
    xTaskCreatePinnedToCore(
                    performReset,   /* Task function. */
                    "Servo",     /* name of task. */
                    1000,       /* Stack size of task */
                    this,        /* parameter of the task */
                    1,           /* priority of the task */
                    &servoResetTask,
                    1);          /* pin task to core 1 */


    openDuration = configuration.openDuration;
    pressedValue = configuration.pressedValue;
    releasedValue = configuration.releasedValue;
    ESP32PWM::allocateTimer(configuration.pwmTimer);
    servo.setPeriodHertz(configuration.pwmFrequency);
    servo.attach(configuration.pin);
    releaseButton();
}

void ServoController::pressButton() {
    shouldPressButton = true;
}

void ServoController::releaseButton() {
    servo.write(releasedValue);
    buttonIsPressed = false;
}

void ServoController::loop(uint32_t millis) {
    if (shouldPressButton) {
        servo.write(pressedValue);
        openingEndTime = millis + openDuration;
        buttonIsPressed = true;
        shouldPressButton = false;
    } else if (buttonIsPressed && millis > openingEndTime) {
        releaseButton();
    }
}


